@@ -1,3 +1,4 @@
+#' @include simRedCard.R
 #' @include formation.R
 #' @include penaltyGoalsProb.R
 #' @include unitedSimClass.R
@@ -23,6 +24,8 @@ utils::globalVariables(c("goalsHome", "goalsAway", "probability"))
 #'
 #' @return Creates an object of the \code{unitedSim} class.
 #' 
+#' @seealso \code{\link{unitedSim}}
+#' 
 #' @examples 
 #' home <- formation(10, NA, c(7,5,3), c(8,8), c(10,10,8))
 #' away <- formation(5, 8, c(8,8), c(10,10), c(10,10,10), 
@@ -33,25 +36,44 @@ utils::globalVariables(c("goalsHome", "goalsAway", "probability"))
 #' unitedSimOne(home, away, r = 100)
 #' 
 #' @export
-unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/15) {
+unitedSimOne <- function(home, away, r, penaltyProb = 0.1, preventGoalGK = 1/14, preventGoalSW = 1/15, 
+                         hardnessMatrix) {
   stopifnot(validObject(home), validObject(away), is(home, "formation"), 
             is(home, "formation"), is.numeric(preventGoalGK), is.numeric(preventGoalSW))
   if (preventGoalGK >= 1/13) stop("preventGoalGK must be smaller than 1/13.")
   if (preventGoalGK < 0) stop("preventGoalGK must be greater than zero.")
   if (preventGoalSW >= 1/13) stop("preventGoalSW must be smaller than 1/13.")
   if (preventGoalSW < 0) stop("preventGoalSW must be greater than zero.")
+  if (penaltyProb < 0 || penaltyProb > 1) stop("Probability for a penalty should be in [0,1]")
+  
+  ## set default value for hardness matrix
+  if (missing(hardnessMatrix)) {
+    hardnessMatrix <- matrix(c(90,10,0,0,0,0,0,0,70,30,0,0,0,0,0,0,50,40,10,
+                               0,0,0,0,0,30,50,20,0,0,0,0,0,20,40,30,10,0,0,
+                               0,0, 10,30,40,20,0,0,0,0,0,20,40,30,10,0,0,0,0,
+                               10,30,40,20,0,0,0,0,0,20,40,30,10,0,0,0,0,10,20,
+                               40,20,10,0,0,0,0,10,40,20,20,10), nrow = 8)
+  }
+  
   if (missing(r)) {
     if (sum(home@hardness) > 1 || sum(away@hardness > 1)) {
       warning("It is recommended to simulate hardness and penalties, calculations are exact for one possible lineup.")
     }
-    
+
     homeLineup <- getLineup(home)
     awayLineup <- getLineup(away)
   
     # simulate red cards
-    homeLineupSim <- simRedCard(home, homeLineup)
-    awayLineupSim <- simRedCard(away, awayLineup)
-  
+    homeLineupSim <- simRedCard(home, homeLineup, hardnessMatrix)
+    awayLineupSim <- simRedCard(away, awayLineup, hardnessMatrix)
+    
+    # save the number of red cards
+    redCardsHome <- homeLineupSim$numberRedCards 
+    redCardsAway <- awayLineupSim$numberRedCards
+    
+    homeLineupSim <- homeLineupSim$lineup
+    awayLineupSim <- awayLineupSim$lineup
+    
     chancesHome <- round((homeLineupSim[3:5] - awayLineupSim[5:3] - 
                           c(0, 0, awayLineupSim[2])) * c(1/4, 1/2, 1) + 0.00001)
     chancesHome <- sum(chancesHome[chancesHome > 0])
@@ -69,8 +91,8 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
     penaltyProbGoalAway <- 1 - (homeLineupSim[1] * 0.05)
   
     # probability distribution of all possible goals by penalties for both teams
-    goalsPenaltyDistrHome <- penaltyGoalsProb(posPenaltiesHome, penaltyProbGoalHome)
-    goalsPenaltyDistrAway <- penaltyGoalsProb(posPenaltiesAway, penaltyProbGoalAway)
+    goalsPenaltyDistrHome <- penaltyGoalsProb(posPenaltiesHome, penaltyProbGoalHome, penaltyProb)
+    goalsPenaltyDistrAway <- penaltyGoalsProb(posPenaltiesAway, penaltyProbGoalAway, penaltyProb)
   
     # possible allocations of the penalties
     penaltyAllocations <- expand.grid(home = 0:posPenaltiesHome, away = 0:posPenaltiesAway)
@@ -131,6 +153,11 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
     finalPossibleResults$tpAway <- ifelse(finalPossibleResults$goalsHome < finalPossibleResults$goalsAway, 1, 
                                         ifelse(finalPossibleResults$goalsHome == finalPossibleResults$goalsAway, 0.5, 
                                                0))
+    
+    # add red cards
+    finalPossibleResults$redCardsHome <- redCardsHome
+    finalPossibleResults$redCardsAway <- redCardsAway
+    
     # output
     output <- new("unitedSim", 
                 results = finalPossibleResults, 
@@ -141,6 +168,8 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
                 winProbabilityHome = round(sum((finalPossibleResults$pointsHome == 3) * finalPossibleResults$probability), digits = 4),
                 winProbabilityAway = round(sum((finalPossibleResults$pointsAway == 3) * finalPossibleResults$probability), digits = 4), 
                 tiedProbability = round(sum((finalPossibleResults$pointsAway == 1) * finalPossibleResults$probability), digits = 4),
+                averageRedCardsHome =  sum(finalPossibleResults$redCardsHome * finalPossibleResults$probability),
+                averageRedCardsAway =  sum(finalPossibleResults$redCardsAway * finalPossibleResults$probability),
                 home = home,
                 away = away)
   
@@ -152,8 +181,15 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
     awayLineup <- getLineup(away)
     simulatedResults <- t(sapply(1:r, function(x) { 
               # simulate red cards
-              homeLineupSim <- simRedCard(home, homeLineup)
-              awayLineupSim <- simRedCard(away, awayLineup)
+              homeLineupSim <- simRedCard(home, homeLineup, hardnessMatrix)
+              awayLineupSim <- simRedCard(away, awayLineup, hardnessMatrix)
+              
+              # save the number of red cards
+              redCardsHome <- homeLineupSim$numberRedCards 
+              redCardsAway <- awayLineupSim$numberRedCards
+              
+              homeLineupSim <- homeLineupSim$lineup
+              awayLineupSim <- awayLineupSim$lineup
               
               chancesHome <- round((homeLineupSim[3:5] - awayLineupSim[5:3] - 
                                       c(0, 0, awayLineupSim[2])) * c(1/4, 1/2, 1))
@@ -164,8 +200,8 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
               chancesAway <- sum(chancesAway[chancesAway > 0])
               
               #  penalties home
-              penaltiesHome <- rbinom(1, sum(away@hardness), 0.1)
-              penaltiesAway <- rbinom(1, sum(home@hardness), 0.1)
+              penaltiesHome <- rbinom(1, sum(away@hardness), penaltyProb)
+              penaltiesAway <- rbinom(1, sum(home@hardness), penaltyProb)
               
               # probability of a goal by penalty
               penaltyProbGoalHome <- 1 - (awayLineupSim[1] * 0.05)
@@ -180,14 +216,14 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
               penaltyGoalsAway <- rbinom(1, penaltiesAway, penaltyProbGoalAway)
               goalsHomeGame <- rbinom(1, chancesHome, probGoalHome)
               goalsAwayGame <- rbinom(1, chancesAway, probGoalAway)
-              c(penaltyGoalsHome + goalsHomeGame, penaltyGoalsAway + goalsAwayGame)
+              c(penaltyGoalsHome + goalsHomeGame, penaltyGoalsAway + goalsAwayGame, redCardsHome, redCardsAway)
       }
     ))
     simulatedResults <- as.data.frame(simulatedResults)
-    colnames(simulatedResults) <- c("goalsHome", "goalsAway")
+    colnames(simulatedResults) <- c("goalsHome", "goalsAway", "redCardsHome", "redCardsAway")
     simulatedResults$probability <- 1/r
     
-    simulatedResults <- ddply(simulatedResults, .(goalsHome, goalsAway), summarize, 
+    simulatedResults <- ddply(simulatedResults, .(goalsHome, goalsAway, redCardsHome, redCardsAway), summarize, 
                                   probability = sum(probability))
     
     # sort results by probability of apprearance
@@ -210,6 +246,10 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
     simulatedResults$tpAway <- ifelse(simulatedResults$goalsHome < simulatedResults$goalsAway, 1, 
                                           ifelse(simulatedResults$goalsHome == simulatedResults$goalsAway, 0.5, 
                                                  0))
+    
+    # change colnames according to the not simulated case
+    simulatedResults <- simulatedResults[, c(1:2, 5:10, 3:4)]
+    
     # output
     output <- new("unitedSimR", 
                   results = simulatedResults, 
@@ -220,6 +260,8 @@ unitedSimOne <- function(home, away, r, preventGoalGK = 1/14, preventGoalSW = 1/
                   winProbabilityHome = round(sum((simulatedResults$pointsHome == 3) * simulatedResults$probability), digits = 4),
                   winProbabilityAway = round(sum((simulatedResults$pointsAway == 3) * simulatedResults$probability), digits = 4), 
                   tiedProbability = round(sum((simulatedResults$pointsAway == 1) * simulatedResults$probability), digits = 4),
+                  averageRedCardsHome =  sum(simulatedResults$redCardsHome * simulatedResults$probability),
+                  averageRedCardsAway =  sum(simulatedResults$redCardsAway * simulatedResults$probability),
                   r = r,
                   home = home,
                   away = away)
